import os

from charmhelpers.core import hookenv
from charmhelpers.core.services.base import (
    manage_ports,
    ServiceManager,
    )
from charmhelpers.core.services import helpers

import actions
import rutabaga_helpers


def render_env_template(config, name):
    return helpers.render_template(
        source='envs/{}.j2'.format(name),
        target='{}/{}'.format(os.path.join(config['base_dir'], 'envs'), name),
        owner=config['code_user'],
        group=config['code_group'])


def render_rsync_template(config):
    if config['log_hosts_allow']:
        return helpers.render_template(
            source='rutabaga-rsync.j2',
            target='/etc/rsyncd.d/010-rutabaga',
            perms=0o644)
    else:
        return lambda _: None


def render_crontab_template(config):
    return helpers.render_template(
        source='rutabaga-cron.j2',
        target='/etc/cron.d/rutabaga',
        perms=0o644)


def manage():

    if hookenv.hook_name() in ('install', 'upgrade-charm'):
        actions.execd_preinstall('rutabaga')
        actions.install_packages('rutabaga')
        actions.create_users('rutabaga')

    if hookenv.hook_name() in ('install', 'upgrade-charm', 'config-changed'):
        actions.deploy_code('rutabaga')

    if hookenv.hook_name() in ('install'):
        actions.create_schema('rutabaga')

    extra_requirements = []

    config = hookenv.config()
    unit_name = hookenv.local_unit().replace('/', '-')
    config['nagios_hostname'] = '{}-{}'.format(
        config['nagios_context'], unit_name)
    manager = ServiceManager([
        {
            'service': 'rutabaga',
            'ports': actions.get_ports(),
            'provided_data': [rutabaga_helpers.RutabagaApiRelation()],
            'required_data': [config] + extra_requirements,
            'data_ready': [
                render_env_template(config, 'DATABASE'),
                render_env_template(config, 'RUTABAGA_LOG_DIR'),
                render_env_template(config, 'SITE_NAME'),
                render_env_template(config, 'MAIN_SITE_ROOT'),
                render_env_template(config, 'ADMIN_API_USERNAME_PREFIX'),
                render_env_template(config, 'ADMIN_API_USERNAME'),
                render_env_template(config, 'ADMIN_API_SECRET'),
                helpers.render_template(
                    source='rutabaga-logrotate.j2',
                    target='/etc/logrotate.d/rutabaga',
                    perms=0o644),
                render_rsync_template(config),
                render_crontab_template(config),
                actions.restart_rsync,
                ],
            'start': [
                actions.publish_wsgi_relations,
                manage_ports,
                ],
            'stop': [
                manage_ports,
                ]
        },
        {
            'service': 'rutabaga-monitoring',
            'required_data': [config],
            'data_ready': [
                helpers.render_template(
                    source='rutabaga-nrpe.j2',
                    target='/etc/nagios/nrpe.d/check_{}.cfg'.format(unit_name),
                ),
                helpers.render_template(
                    source='rutabaga-nagios.j2',
                    target='/var/lib/nagios/export/service__{}-{}.cfg'.format(
                        config['nagios_context'], unit_name),
                ),
            ],
            'data_lost': [
                helpers.render_template(
                    source='rutabaga-nrpe.j2',
                    target='/etc/nagios/nrpe.d/check_{}.cfg'.format(unit_name),
                ),
                helpers.render_template(
                    source='rutabaga-nagios.j2',
                    target='/var/lib/nagios/export/service__{}-{}.cfg'.format(
                        config['nagios_context'], unit_name),
                ),
            ],
            'start': [
                actions.nagios_restart,
                ],
        },
    ])
    manager.manage()
